/*
 * Copyright 2015 Main Street Softworks, Inc. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright notice, this list of
 *     conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list
 *     of conditions and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY MAIN STREET SOFTWORKS INC ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MAIN STREET SOFTWORKS INC OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are those of the
 * authors and should not be interpreted as representing official policies, either expressed
 * or implied, of Main Street Softworks, Inc.
 */

package com.monetra.unitermdemo;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.DialogFragment;
import android.app.FragmentManager;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.GridView;

import com.monetra.uniterm.sdk.UnitermServiceHelper;

import java.util.HashMap;
import java.util.Map;


public class MainActivity extends Activity implements UnitermServiceHelper.UnitermServiceHelperCallbacks {

	// Identifier so we can check if the UnitermServiceHelper has been created and is retained.
    private static final String UNITERMSERVICEHELPER_FRAGMENT = "UnitermServiceHelper_fragment";

    // The SDK object.
    private UnitermServiceHelper mUnitermServiceHelper;

    // Dialogs for status of a transaction.
	// We really should use a DialogFragement instead of checking and recreating
	// dialogs but this is just a simple demo. No need to go to that extent.
    private ProgressDialog mTxnProgressDialog;
    private AlertDialog mTxnResultDialog;

    private SharedPreferences unitermValues;
    private GridView actionsGrid;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        actionsGrid = (GridView) findViewById(R.id.actions_grid);
        String[] actions = getResources().getStringArray(R.array.u_actions);
        ArrayAdapter adapter = new ArrayAdapter(this, R.layout.actions_grid_item, actions);
        actionsGrid.setAdapter(adapter);
        String deviceIdentifier;

		// Check if we have a retained fragment. We don't want to make a new one every time the screen
		// rotates because we'd lose the information about any currently running txnrequest.
        FragmentManager fm = getFragmentManager();
        mUnitermServiceHelper = (UnitermServiceHelper) fm.findFragmentByTag(UNITERMSERVICEHELPER_FRAGMENT);

        if (mUnitermServiceHelper == null) {
            mUnitermServiceHelper = new UnitermServiceHelper();
            fm.beginTransaction().add(mUnitermServiceHelper, UNITERMSERVICEHELPER_FRAGMENT).commit();
            fm.executePendingTransactions();
        }

        unitermValues = PreferenceManager.getDefaultSharedPreferences(this);
        deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        if (deviceIdentifier.equals("")) {
            Intent intent = new Intent(this, SettingsActivity.class);
            startActivity(intent);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        onCreate(null);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.settings:
                Intent intent = new Intent(this, SettingsActivity.class);
                startActivity(intent);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public void takeAction(View view) {
        Button button = (Button) view;
        String buttonText = button.getText().toString().toLowerCase();
        switch (buttonText) {
            case "sale":
                DialogFragment saleDialogFragment = SaleDialogFragment.newInstance();
                saleDialogFragment.show(getFragmentManager(), "dialog");
                break;
            case "quickchip":
                if (mUnitermServiceHelper.IsTxnrequestRunning()) {
                    launchQuickChipFinishDialog();
                } else {
                    startQuickChip();
                }
                break;
            case "version":
                runVersion();
                break;
            case "device info":
                runDeviceInfo();
                break;
            case "device load":
                DialogFragment deviceLoadDialogFragment = DeviceLoadDialogFragment.newInstance();
                deviceLoadDialogFragment.show(getFragmentManager(), "dialog");
                break;
            case "device reboot":
                runDeviceReboot();
                break;
            default:
                break;
        }
    }

    public void startQuickChip() {

        String username = unitermValues.getString("username", "");
        String password = unitermValues.getString("password", "");
        String deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        String deviceType = unitermValues.getString("deviceType", "");

        HashMap<String, String> request = new HashMap<>();

        if (username.isEmpty() || password.isEmpty() || deviceIdentifier.isEmpty() || deviceType.isEmpty()) {
            new AlertDialog.Builder(this).setTitle("TxnRequest").setMessage(getResources().getText(R.string.sale_failure) + "Missing parameters. Need username, password, device, and devicetype").show();
            return;
        }

        request.put("username", username);
        request.put("password", password);
        request.put("u_device", deviceIdentifier);
        request.put("u_devicetype", deviceType);

        request.put("action", "sale");

        setupProgressDialog();
        mTxnProgressDialog.show();
        mUnitermServiceHelper.DoTxnstart(request, true);
    }

    public void finishQuickChip(String amount) {

        String username = unitermValues.getString("username", "");
        String password = unitermValues.getString("password", "");
        String deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        String deviceType = unitermValues.getString("deviceType", "");

        HashMap<String, String> request = new HashMap<>();

        request.put("username", username);
        request.put("password", password);
        request.put("u_device", deviceIdentifier);
        request.put("u_devicetype", deviceType);

        request.put("action", "sale");
        request.put("amount", amount);

        setupProgressDialog();
        mTxnProgressDialog.show();
        mUnitermServiceHelper.DoTxnFinish(request, true);
    }

    public void runVersion() {
        mUnitermServiceHelper.DoVersion();
    }

    public void runDeviceLoad(String type) {

        String username = unitermValues.getString("username", "");
        String password = unitermValues.getString("password", "");
        String deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        String deviceType = unitermValues.getString("deviceType", "");

        HashMap<String, String> request = new HashMap<>();

        if (username.isEmpty() || password.isEmpty() || deviceIdentifier.isEmpty() || deviceType.isEmpty()) {
            new AlertDialog.Builder(this).setTitle("DeviceLoad").setMessage(getResources().getText(R.string.deviceload_failure) + "Missing parameters. Need username, password, device, and devicetype").show();
            return;
        }

        request.put("username", username);
        request.put("password", password);
        request.put("u_device", deviceIdentifier);
        request.put("u_devicetype", deviceType);

        if (!type.isEmpty()) {
            request.put("u_forceload", type);
        }

        setupProgressDialog();
        mTxnProgressDialog.show();
        mUnitermServiceHelper.DoDeviceload(request, false);
    }

    public void runDeviceInfo() {

        String username = unitermValues.getString("username", "");
        String password = unitermValues.getString("password", "");
        String deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        String deviceType = unitermValues.getString("deviceType", "");

        HashMap<String, String> request = new HashMap<>();

        if (username.isEmpty() || password.isEmpty() || deviceIdentifier.isEmpty() || deviceType.isEmpty()) {
            new AlertDialog.Builder(this).setTitle("DeviceInfo").setMessage(getResources().getText(R.string.deviceinfo_failure) + "Missing parameters. Need username, password, device, and devicetype").show();
            return;
        }
        request.put("username", username);
        request.put("password", password);
        request.put("u_device", deviceIdentifier);
        request.put("u_devicetype", deviceType);
        request.put("u_action", "deviceinfo");

        setupProgressDialog();
        mTxnProgressDialog.show();
        mUnitermServiceHelper.DoRawKVS(request, true);
    }

    public void runDeviceReboot() {

        String username = unitermValues.getString("username", "");
        String password = unitermValues.getString("password", "");
        String deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        String deviceType = unitermValues.getString("deviceType", "");

        HashMap<String, String> request = new HashMap<>();

        if (username.isEmpty() || password.isEmpty() || deviceIdentifier.isEmpty() || deviceType.isEmpty()) {
            new AlertDialog.Builder(this).setTitle("DeviceReboot").setMessage(getResources().getText(R.string.devicereboot_failure) + "Missing parameters. Need username, password, device, and devicetype").show();
            return;
        }
        request.put("username", username);
        request.put("password", password);
        request.put("u_device", deviceIdentifier);
        request.put("u_devicetype", deviceType);
        request.put("u_action", "devicereboot");

        setupProgressDialog();
        mTxnProgressDialog.show();
        mUnitermServiceHelper.DoRawKVS(request, true);
    }

    public void TxnrequestCancelResponse(HashMap<String, String> kvs) {
        // Not used
    }

    public void DeviceloadResponse(HashMap<String, String> kvs) {
        mTxnResultDialog = new AlertDialog.Builder(this)
                .setTitle("DeviceLoad")
                .setMessage("u_errorcode: " + kvs.get("u_errorcode") + "\ncode: " + kvs.get("code") + "\nverbiage: " + kvs.get("verbiage"))
                .setPositiveButton("Ok",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }
                )
                .show();

        if (mTxnProgressDialog != null && mTxnProgressDialog.isShowing())
            mTxnProgressDialog.dismiss();
    }

    public void PassthroughResponse(HashMap<String, String> kvs) {
        // Not used
    }

    public void VersionResponse(HashMap<String, String> kvs) {
        new AlertDialog.Builder(this)
            .setTitle("Version")
            .setMessage(kvs.get("verbiage"))
            .setPositiveButton("Done", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    dialog.dismiss();
                }
            })
            .show();
    }

    public void RawKVS(HashMap<String, String> kvs) {
        String response = "";
        for (Map.Entry<String, String> entry : kvs.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            response += key + " = " + value + "\n";
        }
        new AlertDialog.Builder(this)
            .setTitle("Response")
            .setMessage(response)
            .setPositiveButton("Done", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    dialog.dismiss();
                }
            })
            .show();
        if (mTxnProgressDialog != null && mTxnProgressDialog.isShowing())
            mTxnProgressDialog.dismiss();
    }

    public void TxnstartResponse(final HashMap<String, String> kvs) {
        mTxnResultDialog = new AlertDialog.Builder(this)
                .setTitle("TxnStart")
                .setMessage("u_errorcode: " + kvs.get("u_errorcode") + "\ncode: " + kvs.get("code") + "\nverbiage: " + kvs.get("verbiage"))
                .setPositiveButton("Ok",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }
                ).create();
        mTxnResultDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialog) {
                if (kvs.get("u_errorcode").equals("SUCCESS")) {
                    launchQuickChipFinishDialog();
                }
            }
        });
        mTxnResultDialog.show();
        if (mTxnProgressDialog != null && mTxnProgressDialog.isShowing()) {
            mTxnProgressDialog.dismiss();
        }
    }

    public void launchQuickChipFinishDialog() {
        DialogFragment df = QuickChipDialogFragment.newInstance();
        df.show(getFragmentManager(), "dialog");
    }

    public void TxnfinishResponse(HashMap<String, String> kvs) {
        mTxnResultDialog = new AlertDialog.Builder(this)
                .setTitle("TxnStart")
                .setMessage("u_errorcode: " + kvs.get("u_errorcode") + "\ncode: " + kvs.get("code") + "\nverbiage: " + kvs.get("verbiage"))
                .setPositiveButton("Ok",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }
                )
                .show();
        if (mTxnProgressDialog != null && mTxnProgressDialog.isShowing()) {
            mTxnProgressDialog.dismiss();
        }
    }

    public void runSale(String amount) {

        String username = unitermValues.getString("username", "");
        String password = unitermValues.getString("password", "");
        String deviceIdentifier = unitermValues.getString("deviceIdentifier", "");
        String deviceType = unitermValues.getString("deviceType", "");

        HashMap<String, String> request = new HashMap<>();


        if (username.isEmpty() || password.isEmpty() || deviceIdentifier.isEmpty() || deviceType.isEmpty() || amount.isEmpty()) {
            new AlertDialog.Builder(this).setTitle("TxnRequest").setMessage(getResources().getText(R.string.sale_failure) + "Missing parameters. Need username, password, device, devicetype, and amount").show();
            return;
        }

        // Populate our request data.
        // Things like the u_action will be handled by the SDK.
        request.put("username", username);
        request.put("password", password);
        request.put("u_device", deviceIdentifier);
        request.put("u_devicetype", deviceType);

        request.put("action", "sale");
        request.put("amount", amount);

        // Show the progress dialog and start the txnrequest.
        // We'll tell the SDK that we want it to poll the status for us.
        // This way we don't have to poll ourselves and can display the
        // status in the progress dialog each time the status callback
        // is called.
        setupProgressDialog();
        mTxnProgressDialog.show();
        mUnitermServiceHelper.DoTxnrequest(request, true);
    }

    public void TxnrequestStatusResponse(HashMap<String, String> kvs) {
        // If the progress dialog is gone don't try to display anything. At
        // this point we have determined that the dialog shouldn't be recreated
        // if it's not around. Due to threading there is a chance that a txnrequest
        // finishes and a progress status update comes in. For example a status
        // update that the transaction is no longer running because it's finished.
        // If this is the case the response callback would have already dismissed
        // the dialog and we don't want to recreate it.
        if (mTxnProgressDialog == null || !mTxnProgressDialog.isShowing())
            return;

        // The dialog is still showing but the transaction isn't running. Again thread
        // could cause this to come in at the wrong time. Don't show anything to the user
        // to avoid confusion.
        if (!kvs.containsKey("u_errorcode") || !kvs.get("u_errorcode").equalsIgnoreCase("SUCCESS"))
            return;

        // Update the dialog text.
        mTxnProgressDialog.setMessage(kvs.get("verbiage"));

        Button b = mTxnProgressDialog.getButton(DialogInterface.BUTTON_NEGATIVE);
        if (kvs.containsKey("u_cancelable") && kvs.get("u_cancelable").equalsIgnoreCase("yes")) {
            if (b != null) {
                b.setEnabled(true);
            }
        } else {
            b.setEnabled(false);
        }

        // Ensure the dialog is showing.
        if (!mTxnProgressDialog.isShowing())
            mTxnProgressDialog.show();
    }

    public void TxnrequestResponse(HashMap<String, String> kvs) {
        // Inform the user what happened with the transaction.
        mTxnResultDialog = new AlertDialog.Builder(this)
                .setTitle("TxnRequest")
                .setMessage("u_errorcode: " + kvs.get("u_errorcode") + "\ncode: " + kvs.get("code") + "\nverbiage: " + kvs.get("verbiage"))
                .setPositiveButton("Ok",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }
                )
                .show();

        // Dismiss the progress dialog _after_ the blocking alert (response) dialog
        // has finished.
        //
        // When the progress dialog is dismissed before the alert dialog was
        // shown sometimes there was a brief moment when no dialog was shown
        // and the user could press the sale button. We could get around this by
        // disabling the sale button but this demo relies on the dialogs preventing
        // user interaction while a txnrequest is processing.		//
        //
        // This way there is no chance a user can click run sale between
        // hiding and showing the dialogs. That would cause confusion as a transaction
        // gets started as one finishes and dialogs could overlay each other awkwardly.
        if (mTxnProgressDialog != null && mTxnProgressDialog.isShowing())
            mTxnProgressDialog.dismiss();
    }

    public void cancelTransaction() {
        mUnitermServiceHelper.DoTxnrequestCancel();
    }

    private void setupProgressDialog() {
        // Create a progress dialog.
        // We don't allow the dialog to be dismissed on cancel. Instead
        // we start a cancellation and show the progress. This way a
        // txnrequest will finish before we allow the user to run another one.
        mTxnProgressDialog = new ProgressDialog(this);
        mTxnProgressDialog.setTitle("Please Wait...");
        mTxnProgressDialog.setMessage("Working...");
        mTxnProgressDialog.setIndeterminate(true);
        mTxnProgressDialog.setCancelable(false);
        mTxnProgressDialog.setCanceledOnTouchOutside(false);
        mTxnProgressDialog.setButton(DialogInterface.BUTTON_NEGATIVE, getResources().getText(R.string.cancel), (DialogInterface.OnClickListener) null);
        // On cancel we don't dismiss we just send the cancel request. Disable the cancel button so the request
        // is only sent once. Also, so the user gets some indication that it's cancelling.
        mTxnProgressDialog.setOnShowListener(new DialogInterface.OnShowListener() {
            @Override
            public void onShow(DialogInterface dialog) {
                final Button cancelButton = mTxnProgressDialog.getButton(DialogInterface.BUTTON_NEGATIVE);
                cancelButton.setEnabled(false);
                cancelButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        cancelButton.setEnabled(false);
                        cancelTransaction();
                    }
                });
            }
        });
    }

    public void BluetoothlistResponse(String csv) {}
    public void DevicetypesResponse(String csv) {}
}
